<?php
/**
 * Joomla! Content Management System
 *
 * @copyright  Copyright (C) 2005 - 2020 Open Source Matters, Inc. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\CMS\Categories;

/**
 * Representing when a category section cannot be found.
 *
 * @since  3.10.0
 */
class SectionNotFoundException extends \RuntimeException
{
}
